// CustomReplacePlugin.js
const path = require('path');
const { resolveAxiosPath } = require('../utils')

class Webpack4CustomReplacePlugin {
  constructor(options) {
    this.options = options;

    const newMappings = {};
    Object.keys(this.options.mappings).forEach((key) => {
      // 全部转化成绝对路径
      newMappings[path.resolve(this.options.root, key)] = path.resolve(this.options.root, this.options.mappings[key]);
    });

    this.newMappings = newMappings
    console.log('newMapping:', this.newMappings)
  }

  apply(compiler) {
    compiler.hooks.normalModuleFactory.tap('CustomReplacePlugin', (normalModuleFactory) => {
      normalModuleFactory.hooks.beforeResolve.tap('CustomReplacePlugin', (data) => {

        const oldKeys = Object.keys(this.newMappings);

        const currRequest = path.resolve(data.context, data.request);

        if (oldKeys.includes(currRequest)) {
          const distAbsRequest = this.newMappings[currRequest];
          const relativePath = path.relative(data.context, distAbsRequest);
          const distRequest = relativePath.startsWith('.') ? relativePath : `./${relativePath}`;
          console.log('mappings:', data.request, distRequest);
          data.request = distRequest;
        }
      });
    });
  }
}

const axiosRoot = resolveAxiosPath()

const axiosUrlMap = {
  "./lib/adapters/http.js": "./lib/helpers/null.js",
  "./lib/platform/node/index.js": "./lib/platform/browser/index.js",
  "./lib/platform/node/classes/FormData.js": "./lib/helpers/null.js"
}

module.exports = {
  Webpack4CustomReplacePlugin,
  axiosRoot,
  axiosUrlMap
};
