"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var axios_1 = require("axios");
var utils_1 = require("../utils");
var canceller_1 = require("../canceller");
var cancelLastSameRequest_1 = require("../cancelLastSameRequest");
function wxAdapter(config) {
    // adapter接受的config要完全符合axios的配置规范，然后在这里针对不同的引擎做个性化修改
    return new Promise(function (resolve, reject) {
        var requestConfig = {};
        var isAborted = false;
        var requestTask = null;
        var headers = config.headers, data = config.data, params = config.params, url = config.url, timeout = config.timeout, method = config.method, rest = tslib_1.__rest(config, ["headers", "data", "params", "url", "timeout", "method"]);
        var requestHash = (0, utils_1.generateRequestHash)(config);
        var requestHeader = (0, utils_1.deepCopy)(headers);
        delete requestHeader.Referer; // 小程序文档，header中不能有Referer
        requestConfig.header = requestHeader;
        var requestData = data || params; // wx环境，不论get还是post，参数传递的属性名均为data
        requestConfig.data = (0, utils_1.deepCopy)(requestData);
        requestConfig.url = (0, utils_1.buildFullPath)(config.baseURL || '', url || '');
        requestConfig.timeout = timeout;
        requestConfig.method = method === null || method === void 0 ? void 0 : method.toUpperCase();
        requestConfig = tslib_1.__assign(tslib_1.__assign({}, requestConfig), rest);
        requestConfig.success = function (res) {
            var response = {};
            var resData = res.data, statusCode = res.statusCode, header = res.header, restRes = tslib_1.__rest(res, ["data", "statusCode", "header"]);
            response.data = resData;
            response.status = statusCode;
            response.headers = header;
            response.config = config;
            response = tslib_1.__assign(tslib_1.__assign({}, response), restRes);
            var validateStatus = response.config.validateStatus;
            if (!response.status || !validateStatus || validateStatus(response.status)) {
                resolve(response);
            }
            else {
                var errMsg = "Request failed with status code ".concat(response.status);
                reject(new axios_1.AxiosError(errMsg, '', config, requestTask, response));
            }
        };
        requestConfig.fail = function (err) {
            if (isAborted) {
                reject(new axios_1.AxiosError(err.errMsg, axios_1.AxiosError.ERR_CANCELED, config, requestTask));
            }
            reject(new axios_1.AxiosError(err.errMsg, err.errno, config, requestTask));
        };
        // 判断是否已经调用了取消函数
        if ((0, canceller_1.hasCalledCancelerRecord)(requestConfig.cancelKey, requestHash)) {
            // 已经调用过取消函数了，直接抛出错误即可
            reject(new axios_1.AxiosError('请求在发出之前被主动取消！', axios_1.AxiosError.ERR_CANCELED, config, requestTask));
            // 删除记录
            (0, canceller_1.deleteCalledCancellerByHash)(requestConfig.cancelKey, requestHash);
            return;
        }
        requestTask = wx.request(requestConfig);
        // 手动取消请求功能
        // 设置了cancelKey，但是环境不支持AbortController
        if ('undefined' === typeof AbortController) {
            if (requestConfig.cancelKey) {
                (0, canceller_1.addCanceller)(requestConfig.cancelKey, requestHash, function () {
                    isAborted = true;
                    requestTask.abort.call(requestTask);
                    (0, canceller_1.deleteCancellerByHash)(requestConfig.cancelKey, requestHash);
                    reject(new axios_1.AxiosError('请求被取消！', axios_1.AxiosError.ERR_CANCELED, config, requestTask));
                });
            }
            // 取消上一个相同URL的get请求功能
            if (requestConfig.cancelLastSameUrl && requestConfig.method.toLocaleLowerCase() === 'get') {
                (0, cancelLastSameRequest_1.addRequestCanceler)(config.url, requestHash, function () {
                    isAborted = true;
                    requestTask.abort.call(requestTask);
                    (0, cancelLastSameRequest_1.clearCancelerByHash)(config.url, requestHash);
                    reject(new axios_1.AxiosError('请求被取消！', axios_1.AxiosError.ERR_CANCELED, config, requestTask));
                });
            }
        }
    });
}
exports.default = wxAdapter;
wxAdapter.type = 'wx';
//# sourceMappingURL=wx-adapter.js.map