"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxyAjaxCanceler = exports.proxyAxiosInstanceAttrs = exports.addInstanceExtendMethod = exports.normalizeConfig = void 0;
var tslib_1 = require("tslib");
var qs_1 = tslib_1.__importDefault(require("qs"));
var canceller_1 = require("./canceller");
/**
 * 标准化config配置，或根据配置项设置相关的参数
 * 该函数返回的配置对象，应该完全符合axios的配置规范
 * @param {*} config
 */
function normalizeConfig(ctx, config) {
    var _a = tslib_1.__assign(tslib_1.__assign({}, ctx.$options), config), _b = _a.headers, headers = _b === void 0 ? {} : _b, _c = _a.sendFormData, sendFormData = _c === void 0 ? false : _c, _d = _a.data, data = _d === void 0 ? undefined : _d, _e = _a.params, params = _e === void 0 ? {} : _e, _f = _a.baseUrl, baseUrl = _f === void 0 ? '' : _f, _g = _a.baseURL, baseURL = _g === void 0 ? '' : _g, _h = _a.reUseDuration, reUseDuration = _h === void 0 ? 300 : _h, rest = tslib_1.__rest(_a, ["headers", "sendFormData", "data", "params", "baseUrl", "baseURL", "reUseDuration"]);
    // 数据格式为FormData，修改content-type头，以及修改data格式
    var contentType = headers['content-type'] || headers['Content-Type'] || 'application/json';
    headers['content-type'] = sendFormData ? 'application/x-www-form-urlencoded' : contentType;
    var sendData = sendFormData && typeof data === 'object' && data !== null ? qs_1.default.stringify(data) : data;
    // baseUrl，支持两种写法，且用户配置的baseUrl优先级最高
    var bUrl = baseUrl || baseURL || ctx.$baseUrl;
    if (typeof reUseDuration !== 'number' || Number.isNaN(reUseDuration)) {
        throw new Error('ajax配置reUseDuration必须是数字');
    }
    return tslib_1.__assign({ headers: headers, params: params, data: sendData, baseURL: bUrl, reUseDuration: reUseDuration }, rest);
}
exports.normalizeConfig = normalizeConfig;
function addInstanceExtendMethod(ctx) {
    var extend = function (source) {
        Object.keys(source).forEach(function (key) {
            ctx[key] = source[key];
        });
    };
    ctx.extend = extend;
}
exports.addInstanceExtendMethod = addInstanceExtendMethod;
function proxyAxiosInstanceAttrs(target, source) {
    var attrs = ['request', 'interceptors', 'defaults'];
    // const methods: ProxyedAttrs[] = ['request', 'get', 'post', 'delete', 'head', 'options', 'put', 'patch', 'interceptors'];
    attrs.forEach(function (method) {
        Object.defineProperty(target, method, {
            value: source[method],
            writable: false,
        });
    });
}
exports.proxyAxiosInstanceAttrs = proxyAxiosInstanceAttrs;
function proxyAjaxCanceler(ctx) {
    ctx.canceller = new Proxy({}, {
        get: function (target, prop) {
            if (!(0, canceller_1.hasCancelers)(prop)) {
                return function () {
                    var str = "ajax.canceller\u5BF9\u8C61\u4E0A\u4E0D\u5B58\u5728\u540D\u4E3A".concat(prop, "\u7684\u51FD\u6570\uFF01");
                    console.warn(str);
                    return str;
                };
            }
            return function () {
                (0, canceller_1.callCancellerByCancelKey)(prop);
            };
        }
    });
}
exports.proxyAjaxCanceler = proxyAjaxCanceler;
//# sourceMappingURL=helpers.js.map