"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateMethod = void 0;
var tslib_1 = require("tslib");
var helpers_1 = require("./helpers");
var canceller_1 = require("./canceller");
var utils_1 = require("./utils");
var duplicateRequestPrevent_1 = require("./duplicateRequestPrevent");
var cancelLastSameRequest_1 = require("./cancelLastSameRequest");
var postLikeMethods = ['PUT', 'POST', 'DELETE', 'PATCH'];
var generateMethod = function (ajax, method) {
    /**
     *
     * @param url 完整的请求url链接
     * @param options ajax配置
     * @returns 返回一个发送请求的函数
     */
    return function (url, options) {
        /**
         * 发送请求
         * @param params get类请求的url参数 or post类请求的请求体数据
         * @param opts ajax参数，拥有最高优先级
         * @returns 请求数据
         */
        return function (params, opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var mergedOptions, isPostLikeMethod, config, finalOpts, mock, hashKey, currUrl, controller_1, request, e_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        mergedOptions = tslib_1.__assign(tslib_1.__assign({}, options), opts);
                        isPostLikeMethod = postLikeMethods.includes(method);
                        config = tslib_1.__assign(tslib_1.__assign({}, mergedOptions), { params: isPostLikeMethod ? undefined : params, data: isPostLikeMethod ? params : undefined, url: url, method: method });
                        finalOpts = (0, helpers_1.normalizeConfig)(ajax, config);
                        mock = finalOpts.mock;
                        if (mock) {
                            return [2 /*return*/, Promise.resolve(mock)];
                        }
                        hashKey = (0, utils_1.generateRequestHash)(finalOpts);
                        if ((0, duplicateRequestPrevent_1.checkIfHasPendingRequest)(hashKey)) {
                            // 重复，且处于pending状态的请求，复用
                            return [2 /*return*/, (0, duplicateRequestPrevent_1.getCachedRequest)(hashKey)];
                        }
                        // 这个检查应该位于前面复用请求的逻辑后面，不能乱序
                        // 如果需要取消之前处于pending状态相同url请求
                        if (finalOpts.cancelLastSameUrl) {
                            currUrl = finalOpts.url;
                            // 执行所有缓存的url相关的取消函数
                            (0, cancelLastSameRequest_1.execCanceler)(currUrl);
                        }
                        // 缓存当前请求的取消函数
                        if (typeof AbortController !== 'undefined') {
                            controller_1 = new AbortController();
                            finalOpts.signal = controller_1.signal;
                            if (typeof mergedOptions.cancelKey !== 'undefined') {
                                (0, canceller_1.addCanceller)(mergedOptions.cancelKey, hashKey, function () {
                                    controller_1.abort.call(controller_1);
                                    // 删除挂载在ajax上的请求中断函数
                                    (0, canceller_1.deleteCancellerByHash)(mergedOptions.cancelKey, hashKey);
                                });
                            }
                            // 缓存当前请求的取消函数，假如后续有相同的url请求需要取消该请求，可以通过这里缓存的函数实现（只针对get请求）
                            if (finalOpts.cancelLastSameUrl && finalOpts.method.toLocaleLowerCase() === 'get') {
                                (0, cancelLastSameRequest_1.addRequestCanceler)(finalOpts.url, hashKey, function () {
                                    controller_1.abort.call(controller_1);
                                    (0, cancelLastSameRequest_1.clearCancelerByHash)(finalOpts.url, hashKey);
                                });
                            }
                        }
                        else {
                            // 环境如果不支持AbortController，取消函数挂载到ajax只能交给adapter里面去实现了
                            finalOpts.signal = undefined;
                            // 因为异步的关系(axios中发出请求之前，对拦截函数处理可能是异步的)，逻辑可能还没走到adapter中去，取消函数就已经被调用了
                            // 此时取消函数还是undefined，所以这里提供默认取消函数
                            if (typeof mergedOptions.cancelKey !== 'undefined') {
                                (0, canceller_1.addCanceller)(mergedOptions.cancelKey, hashKey, function () {
                                    // 默认函数将已经调用的取消函数名称记录下来，在adapter中走到发送请求的逻辑时，直接中断即可
                                    (0, canceller_1.addCalledCanceller)(mergedOptions.cancelKey, hashKey, true);
                                    (0, canceller_1.deleteCancellerByHash)(mergedOptions.cancelKey, hashKey);
                                });
                            }
                        }
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, 4, 5]);
                        request = ajax.request(finalOpts);
                        (0, duplicateRequestPrevent_1.addPendingRequest)(hashKey, request, finalOpts.reUseDuration);
                        return [4 /*yield*/, request];
                    case 2: return [2 /*return*/, _a.sent()];
                    case 3:
                        e_1 = _a.sent();
                        return [2 /*return*/, Promise.reject(e_1)];
                    case 4:
                        // 请求成功，删除ajax.canceller上对应的取消函数
                        (0, canceller_1.deleteCancellerByHash)(finalOpts.cancelKey, hashKey);
                        // 删除用于取消相同URL请求功能而缓存的取消函数
                        (0, cancelLastSameRequest_1.clearCancelerByHash)(finalOpts.url, hashKey);
                        // 删除缓存的请求（用于复用finalOpts.reUseDuration内完全相同的多个请求）
                        (0, duplicateRequestPrevent_1.clearPendingRequest)(hashKey);
                        return [7 /*endfinally*/];
                    case 5: return [2 /*return*/];
                }
            });
        }); };
    };
};
exports.generateMethod = generateMethod;
//# sourceMappingURL=methods.js.map