"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRequestHash = exports.getType = exports.deepCopy = exports.buildFullPath = exports.combineURLs = exports.isAbsoluteURL = void 0;
var tslib_1 = require("tslib");
var hash_sum_1 = tslib_1.__importDefault(require("hash-sum"));
function isAbsoluteURL(url) {
    // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
    // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
    // by any combination of letters, digits, plus, period, or hyphen.
    return /^([a-z][a-z\d+-.]*:)?\/\//i.test(url);
}
exports.isAbsoluteURL = isAbsoluteURL;
function combineURLs(baseURL, relativeURL) {
    return relativeURL
        ? "".concat(baseURL.replace(/\/+$/, ''), "/").concat(relativeURL.replace(/^\/+/, ''))
        : baseURL;
}
exports.combineURLs = combineURLs;
function buildFullPath(baseURL, requestedURL) {
    if (baseURL && !isAbsoluteURL(requestedURL)) {
        return combineURLs(baseURL, requestedURL);
    }
    return requestedURL;
}
exports.buildFullPath = buildFullPath;
function deepCopy(obj) {
    var target;
    if (typeof obj === 'object' && obj !== null) {
        if (Array.isArray(obj)) {
            target = [];
            obj.forEach(function (elem) { return target.push(deepCopy(elem)); });
        }
        else {
            target = {};
            Object.keys(obj).forEach(function (key) {
                target[key] = deepCopy(obj[key]);
            });
        }
    }
    else {
        target = obj;
    }
    return target;
}
exports.deepCopy = deepCopy;
function getType(obj) {
    var type = typeof obj;
    if (type !== 'object')
        return type;
    return Object.prototype.toString.call(obj)
        .replace(/^\[object\s+(\S+)\]$/, '$1')
        .toLowerCase();
}
exports.getType = getType;
/**
 * 获取请求的标识，相同的请求配置有相同的标识
 * @param config 经过标准化的ajax请求配置
 * @returns 该请求的hash key
 */
function generateRequestHash(config) {
    var _a = config.url, url = _a === void 0 ? '' : _a, _b = config.method, method = _b === void 0 ? '' : _b, _c = config.params, params = _c === void 0 ? {} : _c, _d = config.data, data = _d === void 0 ? {} : _d, _e = config.baseURL, baseURL = _e === void 0 ? '' : _e;
    // 合并请求参数
    var combinedData = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, params), data), { baseURL: baseURL, url: url, method: method });
    // 创建一个哈希
    var hash = (0, hash_sum_1.default)(combinedData);
    return hash;
}
exports.generateRequestHash = generateRequestHash;
//# sourceMappingURL=utils.js.map